// SPDX-FileCopyrightText: 2012 Giovanni Campagna <gcampagna@src.gnome.org>
// SPDX-FileCopyrightText: 2014 Florian Müllner <fmuellner@gnome.org>
//
// SPDX-License-Identifier: GPL-2.0-or-later

/* exported WorkspacesPage */
const { Gio, GObject, Gtk, Pango } = imports.gi;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();

const Gettext = imports.gettext.domain(Me.metadata['gettext-domain']);
const _ = Gettext.gettext;

const PreferencesGroup = GObject.registerClass({
    Properties: {
        'title': GObject.ParamSpec.string(
            'title', '', '',
            GObject.ParamFlags.READWRITE,
            null),
    },
}, class PreferencesGroup extends Gtk.Box {
    _init(params) {
        super._init({
            ...params,
            orientation: Gtk.Orientation.VERTICAL,
        });

        const titleLabel = new Gtk.Label({
            halign: Gtk.Align.START,
        });
        titleLabel.add_css_class('heading');

        this.bind_property('title',
            titleLabel, 'label',
            GObject.BindingFlags.SYNC_CREATE);
        this.append(titleLabel);

        this._list = new Gtk.ListBox({
            selection_mode: Gtk.SelectionMode.NONE,
            valign: Gtk.Align.START,
            show_separators: true,
        });
        this._list.add_css_class('boxed-list');
        this.append(this._list);

        this._list.connect('row-activated',
            (l, row) => row.activate());
    }

    get title() {
        return this._title || '';
    }

    set title(title) {
        if (this._title === title)
            return;

        this._title = title;
        this.notify('title');
    }

    add(child) {
        this._list.append(child);
    }

    remove(child) {
        this._list.remove(child);
    }
});

const ActionRow = GObject.registerClass({
    Properties: {
        'title': GObject.ParamSpec.string(
            'title', '', '',
            GObject.ParamFlags.READWRITE,
            null),
        'subtitle': GObject.ParamSpec.string(
            'subtitle', '', '',
            GObject.ParamFlags.READWRITE,
            null),
        'activatable-widget': GObject.ParamSpec.object(
            'activatable-widget', '', '',
            GObject.ParamFlags.READWRITE,
            Gtk.Widget),
    },
}, class ActionRow extends Gtk.ListBoxRow {
    _init(params) {
        super._init(params);

        const provider = new Gtk.CssProvider();
        provider.load_from_data(`* {
            border-spacing: 6px 6px;
            min-height: 40px;
            margin-left: 12px;
            margin-right: 12px;
        }`, -1);

        const mainBox = new Gtk.Box({
            valign: Gtk.Align.CENTER,
            hexpand: false,
        });
        mainBox.get_style_context().add_provider(provider,
            Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION);
        this.set_child(mainBox);

        this._prefixes = new Gtk.Box({
            visible: false,
        });
        mainBox.append(this._prefixes);

        const titleBox = new Gtk.Box({
            orientation: Gtk.Orientation.VERTICAL,
            valign: Gtk.Align.CENTER,
            hexpand: true,
        });
        mainBox.append(titleBox);

        this._suffixes = new Gtk.Box({
            visible: false,
        });
        mainBox.append(this._suffixes);

        const titleLabel = new Gtk.Label({
            lines: 0,
            wrap: true,
            wrap_mode: Pango.WrapMode.WORD_CHAR,
            xalign: 0,
        });

        this.bind_property('title',
            titleLabel, 'label',
            GObject.BindingFlags.SYNC_CREATE);
        titleBox.append(titleLabel);

        const subtitleLabel = new Gtk.Label({
            ellipsize: Pango.EllipsizeMode.NONE,
            lines: 0,
            wrap: true,
            wrap_mode: Pango.WrapMode.WORD_CHAR,
            xalign: 0,
            visible: this.subtitle.length > 0,
        });
        subtitleLabel.add_css_class('dim-label');

        this.bind_property('subtitle',
            subtitleLabel, 'label',
            GObject.BindingFlags.SYNC_CREATE);
        titleBox.append(subtitleLabel);
    }

    get title() {
        return this._title || '';
    }

    set title(title) {
        if (this._title === title)
            return;

        this._title = title;
        this.notify('title');
    }

    get subtitle() {
        return this._subtitle || '';
    }

    set subtitle(subtitle) {
        if (this._subtitle === subtitle)
            return;

        this._subtitle = subtitle;
        this.notify('subtitle');
    }

    get activitable_widget() {
        return this._activatableWidget;
    }

    set activatable_widget(widget) {
        if (this._activatableWidget === widget)
            return;

        this._activatableWidget = widget;
        this.notify('activatable-widget');
    }

    add_prefix(child) {
        this._prefixes.append(child);
        this._prefixes.set_visible(true);
    }

    add_suffix(child) {
        this._suffixes.append(child);
        this._suffixes.set_visible(true);
    }

    activate() {
        if (this._activatableWidget)
            this._activatableWidget.mnemonic_activate(false);
    }
});

const SpinRow = GObject.registerClass({
    Properties: {
        'adjustment': GObject.ParamSpec.object(
            'adjustment', '', '',
            GObject.ParamFlags.READWRITE,
            Gtk.Adjustment),
        'value': GObject.ParamSpec.double(
            'value', '', '',
            GObject.ParamFlags.READWRITE,
            -Infinity, Infinity, 0),
    },
}, class SpinRow extends ActionRow {
    _init(params) {
        this._spinButton = new Gtk.SpinButton({
            valign: Gtk.Align.CENTER,
            hexpand: true,
            xalign: 1,
        });
        super._init(params);
        this.add_suffix(this._spinButton);
    }

    get adjustment() {
        return this._spinButton.get_adjustment();
    }

    set adjustment(adj) {
        if (this.adjustment === adj)
            return;

        this._spinButton.set_adjustment(adj);
        this.notify('adjustment');
    }

    get value() {
        return this._spinButton.get_value();
    }

    set value(value) {
        const EPSILON = 0.005;

        if (Math.abs(this.value, value) < EPSILON)
            return;

        this._spinButton.set_value(value);
        this.notify('value');
    }
});

const GeneralGroup = GObject.registerClass(
class GeneralGroup extends PreferencesGroup {
    _init() {
        super._init({
            title: _('Indicator'),
        });

        const previewCheck = new Gtk.CheckButton();
        const previewRow = new ActionRow({
            title: _('Previews'),
            activatable_widget: previewCheck,
        });
        previewRow.add_prefix(previewCheck);
        this.add(previewRow);

        const nameCheck = new Gtk.CheckButton({
            group: previewCheck,
        });
        const nameRow = new ActionRow({
            title: _('Workspace Name'),
            activatable_widget: nameCheck,
        });
        nameRow.add_prefix(nameCheck);
        this.add(nameRow);

        const settings = ExtensionUtils.getSettings();
        if (settings.get_boolean('embed-previews'))
            previewCheck.active = true;
        else
            nameCheck.active = true;

        settings.bind('embed-previews',
            previewCheck, 'active',
            Gio.SettingsBindFlags.DEFAULT);
    }
});

const BehaviorGroup = GObject.registerClass(
class BehaviorGroup extends PreferencesGroup {
    _init() {
        super._init({
            title: _('Behavior'),
        });

        const dynamicCheck = new Gtk.CheckButton();
        const dynamicRow = new ActionRow({
            title: _('Dynamic'),
            subtitle: _('Automatically removes empty workspaces.'),
            activatable_widget: dynamicCheck,
        });
        dynamicRow.add_prefix(dynamicCheck);
        this.add(dynamicRow);

        const fixedCheck = new Gtk.CheckButton({
            group: dynamicCheck,
        });
        const fixedRow = new ActionRow({
            title: _('Fixed Number'),
            subtitle: _('Specify a number of permanent workspaces.'),
            activatable_widget: fixedCheck,
        });
        fixedRow.add_prefix(fixedCheck);
        this.add(fixedRow);

        const adjustment = new Gtk.Adjustment({
            lower: 1,
            step_increment: 1,
            value: 4,
            upper: 36, // hard limit in mutter
        });
        const numRow = new SpinRow({
            title: _('Number of Workspaces'),
            adjustment,
        });
        this.add(numRow);

        const mutterSettings = new Gio.Settings({
            schema_id: 'org.gnome.mutter',
        });

        if (mutterSettings.get_boolean('dynamic-workspaces'))
            dynamicCheck.active = true;
        else
            fixedCheck.active = true;

        mutterSettings.bind('dynamic-workspaces',
            dynamicCheck, 'active',
            Gio.SettingsBindFlags.DEFAULT);

        const desktopSettings = new Gio.Settings({
            schema_id: 'org.gnome.desktop.wm.preferences',
        });

        desktopSettings.bind('num-workspaces',
            numRow, 'value',
            Gio.SettingsBindFlags.DEFAULT);

        fixedCheck.bind_property('active',
            numRow, 'sensitive',
            GObject.BindingFlags.SYNC_CREATE);
    }
});

var WorkspacesPage = GObject.registerClass(
class WorkspacesPage extends Gtk.ScrolledWindow {
    _init() {
        super._init({
            hscrollbar_policy: Gtk.PolicyType.NEVER,
            vexpand: true,
        });

        const box = new Gtk.Box({
            orientation: Gtk.Orientation.VERTICAL,
            halign: Gtk.Align.CENTER,
            spacing: 12,
            margin_top: 36,
            margin_bottom: 36,
            margin_start: 36,
            margin_end: 36,
        });
        this.set_child(box);

        box.append(new GeneralGroup());
        box.append(new BehaviorGroup());
    }
});
